/**
 * Created : Mar 1, 2012
 */
package com.ivy.homework.phonebook;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ivy.homework.phonebook.impl.AddressBookServiceImpl;
import com.ivy.homework.phonebook.impl.AddressBookViewCmdImpl;
import com.ivy.homework.phonebook.service.AddressBookService;
import com.ivy.homework.phonebook.service.AddressBookView;
import com.ivy.homework.phonebook.service.ProcessorService;

/**
 * Address Book Command handling thread.<br>
 * "quit" to end the thread.
 * 
 * @author eyandai
 */
public final class AddressBookExecutor {
    private static final Logger LOGGER = LoggerFactory.getLogger(AddressBookExecutor.class);
    private final ExecutorService EXECUTOR_SERVICE = Executors.newSingleThreadExecutor();
    private ProcessorService processor;
    private String command = null;

    public AddressBookExecutor() {
        super();
        AddressBookView view = new AddressBookViewCmdImpl();
        AddressBookService addressBook = new AddressBookServiceImpl();
        processor = new Processor(addressBook, view);
    }

    /**
     * set command processor
     * 
     * @param processor
     *            the command processor
     */
    public void setProcessor(ProcessorService processor) {
        this.processor = processor;
    }

    /**
     * start to receive command
     */
    public void work() {
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
            task(br);
            br.close();
            EXECUTOR_SERVICE.shutdown();
        } catch (IOException e) {
            LOGGER.error(e.toString());
        }
    }

    /**
     * receive a command, process it and wait for another command.<br>
     * "quit" to end the loop.<br>
     * 
     * @param br
     * @throws IOException
     */
    public void task(BufferedReader br) throws IOException {
        command = br.readLine();
        while (command != null) {
            if (command.contains(Constants.QUIT)) {
                processor.stop();
                LOGGER.debug("quit command received");
                return;
            } else {
                EXECUTOR_SERVICE.execute(new Runnable() {
                    public void run() {
                        processor.execute(command);
                    }
                });
            }
            command = br.readLine();
        }
    }

    public static void main(String args[]) {
        AddressBookExecutor addressBookExecutor = new AddressBookExecutor();
        addressBookExecutor.work();
    }

}
