/**
 * Created : Dec 29, 2011
 */
package com.ivy.homework.phonebook.exception;

/**
 * <tt>ErrorCode</tt> represent one specific error, the digit code should be predefined in system according to the rule.
 * <p>
 * 
 * And according to the rule, each error code have the information, including:
 * <p>
 * 
 * <li>Area: Which area generated the error, eg, platform</li>
 * <li>Module Name: Which service generated the error</li>
 * <li>Error type: system internal or logical</li>
 * <p>
 * 
 * @author eyanpen
 * 
 */
public final class ErrorCode {

    private final long code;
    private final String description;

    /**
     * 
     * @param errorCode
     *            code of error
     * @param errorDescription
     *            description of error
     */
    public ErrorCode(long errorCode, String errorDescription) {
        this.code = errorCode;
        this.description = errorDescription;
    }

    /**
     * 
     * @param errorCode
     *            code of error
     * @param errorDescription
     *            description of error
     */
    public ErrorCode(String errorCode, String errorDescription) {
        this(Long.parseLong(errorCode), errorDescription);
    }

    /**
     * Get the error code
     * 
     * @param errorCode
     *            code of error
     */
    public long getCode() {
        return this.code;
    }

    /**
     * Get the error description
     * 
     * @return the errorDescription.
     */
    public String getDescription() {
        return this.description;
    }

    /**
     * phone number is invalid. expect phone number is a 11 number
     */
    public static ErrorCode INVALID_PHONE_NUMBER = new ErrorCode(100, "phone number is invalid");

    /**
     * phone number is invalid. expect phone number
     */
    public static ErrorCode PHONE_NUMBER_MISSING = new ErrorCode(100, "phone number argument expected");
}
