package com.ivy.homework.phonebook.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ivy.homework.phonebook.bean.Contact;
import com.ivy.homework.phonebook.service.AddressBookService;
import com.ivy.homework.phonebook.service.AddressKeeperService;

/**
 * Address Book. <br>
 * it is used in cli mode.
 * 
 * @author eyandai
 */
public final class AddressBookServiceImpl implements AddressBookService {
    private static final Logger LOGGER = LoggerFactory.getLogger(AddressBookServiceImpl.class);
    private Map<String, Contact> contactMap = new HashMap<String, Contact>();
    private AddressKeeperService keeperService;

    public AddressBookServiceImpl() {
        super();
        keeperService = new AddressKeeperServiceXMLImpl();
        open();
    }

    public void setKeeperService(AddressKeeperService keeperService) {
        this.keeperService = keeperService;
    }

    /**
     * open the address book
     */
    public void open() {
        contactMap = keeperService.getContacts();
        LOGGER.info("address book is opened");
    }

    /**
     * close the address book
     */
    public void close() {
        save();
        contactMap.clear();
        LOGGER.info("address book is closed");
    }

    /**
     * save the address book before exit
     */
    public void save() {
        keeperService.saveContacts(contactMap);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#newcontact(com.ivy.homework.phonebook.impl.Contact)
     */
    public void newContact(Contact contact) {
        contactMap.put(contact.getName(), contact);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#deletecontact(java.lang.String)
     */
    public void deleteContact(String name) {
        contactMap.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#setcontact(com.ivy.homework.phonebook.impl.Contact)
     */
    public void setContact(Contact contact) {
        contactMap.put(contact.getName(), contact);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#searchByName()
     */
    public List<Contact> searchByName(String name) {
        List<Contact> contacts = new ArrayList<Contact>();
        for (String key : contactMap.keySet()) {
            if (key.contains(name)) {
                contacts.add(contactMap.get(key));
            }
        }
        return contacts;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#getAll()
     */
    public List<Contact> getAll() {
        List<Contact> contacts = new ArrayList<Contact>();
        for (String key : contactMap.keySet()) {
            contacts.add(contactMap.get(key));
        }
        return contacts;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#searchByAddress(java.lang.String)
     */
    public List<Contact> searchByAddress(String address) {
        List<Contact> contacts = new ArrayList<Contact>();
        for (String key : contactMap.keySet()) {
            if (contactMap.get(key).getAddress().contains(address)) {
                contacts.add(contactMap.get(key));
            }
        }
        return contacts;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.AddressBookManager#searchByPhoneNumber(java.lang.String)
     */
    public List<Contact> searchByPhoneNumber(String phoneNumber) {
        List<Contact> contacts = new ArrayList<Contact>();
        for (String key : contactMap.keySet()) {
            if (contactMap.get(key).getPhoneNumber().contains(phoneNumber)) {
                contacts.add(contactMap.get(key));
            }
        }
        return contacts;
    }
}
