/**
 * Created : Mar 1, 2012
 */
package com.ivy.homework.phonebook.impl;

import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ivy.homework.phonebook.Constants;
import com.ivy.homework.phonebook.bean.Contact;
import com.ivy.homework.phonebook.service.AddressKeeperService;

/**
 * Address book keeper service impl.<br>
 * 
 * @author eyandai
 */
public final class AddressKeeperServiceXMLImpl implements AddressKeeperService {

    private static final Logger LOGGER = LoggerFactory.getLogger(AddressKeeperServiceXMLImpl.class);
    private DocumentBuilder builder = null;
    private String fileName = "phonebook.xml";
    private File file = new File(fileName);

    public AddressKeeperServiceXMLImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.service.AddressKeeperService#getContacts()
     */
    public Map<String, Contact> getContacts() {
        Map<String, Contact> map = new HashMap<String, Contact>();
        Document document = getDocument();
        if (document != null) {
            Element rootElement = document.getDocumentElement();
            NodeList nodeList = rootElement.getChildNodes();
            for (int i = 0; i < nodeList.getLength(); i++) {
                Node node = nodeList.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    String name = ((Element) node).getElementsByTagName(Constants.NAME).item(0).getTextContent();
                    String address = ((Element) node).getElementsByTagName(Constants.ADDRESS).item(0).getTextContent();
                    String phonenumber = ((Element) node).getElementsByTagName(Constants.PHONE).item(0).getTextContent();
                    Contact contact = new Contact();
                    contact.setAddress(address);
                    contact.setName(name);
                    contact.setPhoneNumber(phonenumber);
                    map.put(name, contact);
                }
            }
        }
        LOGGER.info("get saved contacts");
        return map;
    }

    private Document getDocument() {
        Document document = null;
        try {
            getDocumentBuilder();
            document = builder.parse(file.getAbsolutePath());
        } catch (Exception e) {
            LOGGER.error("error happen when get Document " + e.toString());
        }
        return document;
    }

    private DocumentBuilder getDocumentBuilder() {
        try {
            if (builder == null) {
                builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            }
        } catch (ParserConfigurationException e) {
            LOGGER.error("error happen when get Document Builder " + e.toString());
        }
        return builder;
    }

    /**
     * get the file name to store contacts
     * 
     * @return
     */
    public String getFileName() {
        return fileName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ivy.homework.phonebook.impl.service.AddressKeeperService#saveContacts(java.util.Map)
     */
    public void saveContacts(Map<String, Contact> map) {
        getDocumentBuilder();
        Document document = builder.newDocument();
        Element rootElement = document.createElement(Constants.ADDRESS_BOOK);
        if (map.size() == 0) {
            return;
        }
        for (String name : map.keySet()) {
            Contact contact = map.get(name);
            Element nameNode = document.createElement(Constants.NAME);
            nameNode.setTextContent(contact.getName());
            Element addressNode = document.createElement(Constants.ADDRESS);
            addressNode.setTextContent(contact.getAddress());
            Element phoneNode = document.createElement(Constants.PHONE);
            phoneNode.setTextContent(contact.getPhoneNumber());
            Element contactElement = document.createElement(Constants.CONTACT);
            contactElement.appendChild(nameNode);
            contactElement.appendChild(addressNode);
            contactElement.appendChild(phoneNode);
            rootElement.appendChild(contactElement);
        }
        document.appendChild(rootElement);
        saveDocument(document);
    }

    private void saveDocument(Document document) {
        try {
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            DOMSource source = new DOMSource(document);
            StreamResult result = new StreamResult(new StringWriter());
            transformer.transform(source, result);

            FileWriter fileWriter = new FileWriter(file.getAbsolutePath());
            fileWriter.write(result.getWriter().toString());
            fileWriter.flush();
            fileWriter.close();

        } catch (Exception e) {
            LOGGER.error("error happen when save document " + e.toString());
        }
        LOGGER.info(" saved contacts");
    }

    /**
     * set the file name to store contacts
     * 
     * @param fileName
     */
    public void setFileName(String fileName) {
        this.fileName = fileName;
        file = new File(fileName);
    }
}
