/**
 * Created : Mar 1, 2012
 */
package com.ivy.homework.phonebook.service;

import java.util.List;

import com.ivy.homework.phonebook.bean.Contact;

/**
 * address book manager.<br>
 * address book service provider
 * 
 * @author eyandai
 */
public interface AddressBookService {

    /**
     * open a address book
     */
     void open();

    /**
     * close a address book, and save it .
     */
     void close();

    /**
     * create a contact
     * 
     * @param contact
     *            contact to create
     */
     void newContact(Contact contact);

    /**
     * delete a contact
     * 
     * @param name
     *            the name of contact
     */
     void deleteContact(String name);

    /**
     * modify a contact
     * 
     * @param contact
     *            new contact
     */
     void setContact(Contact contact);

    /**
     * search by name
     * 
     * @param name
     *            name or partial of name
     * @return contacts which contain the name
     */
     List<Contact> searchByName(String name);

    /**
     * search by address
     * 
     * @param address
     *            address or partical of address
     * @return contacts which contain the address
     */
     List<Contact> searchByAddress(String address);

    /**
     * search by phoneNumber
     * 
     * @param phoneNumber
     *            phone number or partial of phone number
     * @return contacts which contain the phoneNumber
     */
     List<Contact> searchByPhoneNumber(String phoneNumber);

    /**
     * get all the contacts
     * 
     * @return all the contacts
     */
     List<Contact> getAll();

    /**
     * set address book keeper
     * 
     * @param addressKeeperService
     *            keeper which hold address book
     */
     void setKeeperService(AddressKeeperService addressKeeperService);
}
