/**
 * Created : Mar 2, 2012
 */
package com.ivy.homework.phonebook;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import org.junit.Before;
import org.junit.Test;

import com.ivy.homework.phonebook.bean.Contact;
import com.ivy.homework.phonebook.exception.MyException;
import com.ivy.homework.phonebook.service.AddressBookService;
import com.ivy.homework.phonebook.service.AddressBookView;
import com.ivy.homework.phonebook.service.ProcessorService;

/**
 * 
 * @author eyandai
 */
public class ProcessorTest {

    private ProcessorService processor;
    private AddressBookService addressBookServiceImpl;
    private AddressBookView view;

    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception {
        addressBookServiceImpl = mock(AddressBookService.class);
        view = mock(AddressBookView.class);
        processor = new Processor(addressBookServiceImpl, view);

    }

    /**
     * Test method for {@link com.ivy.homework.phonebook.impl.Processor#execute(java.lang.String)}.
     */
    @Test
    public void testExecuteInvalid() throws Exception {
        processor.execute("save -n ");
        verify(addressBookServiceImpl, times(0)).deleteContact(anyString());
        verify(addressBookServiceImpl, times(0)).newContact(any(Contact.class));

        processor.execute("save -n ivy");
        verify(addressBookServiceImpl, times(0)).deleteContact(anyString());
        verify(addressBookServiceImpl, times(0)).newContact(any(Contact.class));
    }

    /**
     * Test method for {@link com.ivy.homework.phonebook.impl.Processor#set(org.apache.commons.cli.CommandLine)}.
     */
    @Test
    public void testSet() throws Exception {
        processor.execute("create -n ivy -a address1 -p 13378954562 ");
        processor.execute("set -n ivy -a address1 -p 13378954562 ");

        verify(addressBookServiceImpl, times(1)).setContact(any(Contact.class));
    }

    /**
     * Test method for {@link com.ivy.homework.phonebook.impl.Processor#create(org.apache.commons.cli.CommandLine)}.
     */
    @Test
    public void testCreate() throws Exception {
        processor.execute("create -n ivy -a address1 -p 13378954562 ");
        verify(addressBookServiceImpl, times(1)).newContact(any(Contact.class));
    }

    /**
     * Test method for {@link com.ivy.homework.phonebook.impl.Processor#search(org.apache.commons.cli.CommandLine)}.
     */
    @Test
    public void testSearch() throws Exception {
        processor.execute("create -n ivy -a address1 -p 13568974567 ");
        processor.execute("search -n ivy  ");
        processor.execute("search -a address  ");
        processor.execute("search -p 123  ");
        processor.execute("search ");
        verify(addressBookServiceImpl, times(1)).searchByAddress(anyString());
        verify(addressBookServiceImpl, times(1)).searchByName(anyString());
        verify(addressBookServiceImpl, times(1)).searchByPhoneNumber(anyString());
    }

    /**
     * Test method for {@link com.ivy.homework.phonebook.impl.Processor#delete(org.apache.commons.cli.CommandLine)}.
     */
    @Test
    public void testDelete() throws Exception {
        processor.execute("create -n ivy -a address1 -p 13568974567 ");
        processor.execute("delete ");
        processor.execute("delete -n ivy");
        verify(addressBookServiceImpl, times(1)).deleteContact(anyString());
    }

    /**
     * Test method for {@link com.ivy.homework.phonebook.impl.Processor#help()}.
     */
    @Test
    public void testHelp() {
        processor.help();
    }

    /**
     * test validate a valid phone number
     * 
     * @throws Exception
     */
    @Test
    public void validateGoodPhoneNumber() throws Exception {
        processor.validatePhoneNumber("13568974567");
    }

    /**
     * test validate an invalid phone number
     * 
     * @throws Exception
     *             expected exception is MyException
     */
    @Test(expected = MyException.class)
    public void testInvalidPhoneNumber() throws Exception {
        processor.validatePhoneNumber("123");
    }

    /**
     * test stop proccessor
     * 
     */
    @Test
    public void testStop() {
        processor.stop();
    }
}
